/** 
 * Copyright (C) 2008 Alan Ross, ( http://lab.adjazent.com )
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
package ar.particlepainter.paint 
{
	import ar.particlepainter.particles.BloodParticle;
	
	import flash.display.BitmapData;	

	/**
	 * @author Alan Ross
	 */
	public class BloodPaint implements IPaint
	{
		private const NUM_PARTICLES: int = 3000;
		
		private const PI2: Number = Math.PI * 2;
		
		private var _canvas: BitmapData;
		
		private var _radius: int;
		
		private var _particles: BloodParticle;

		/**
		 * Create a new <code>BloodPaint</code> object.
		 */
		public function BloodPaint( canvas: BitmapData, radius: int = 100 )
		{
			_canvas = canvas;
			
			_radius = radius;
			
			var p: BloodParticle;
			
			p = _particles = new BloodParticle();
			
			var n: int = NUM_PARTICLES;
			
			while( --n > -1 )
			{
				p = p.next = new BloodParticle();
				p.color = 0xCC0000;
			}
		}
		
		/**
		 * @inheritDoc
		 */
		public function paint( mouseX: int, mouseY: int, mouseDown: Boolean ): void
		{
			var temp: Number;
			
			var p: BloodParticle = _particles.next;
			
			do
			{	
				--p.restLife;
				
				if( p.restLife < 0 )
				{
					if( mouseDown )
					{
						temp = Math.random() * PI2;
						
						p.distToOrigin = ( _radius * Math.random() ) / p.size;
						
						p.x = mouseX + ( Math.sin( temp ) * p.distToOrigin );
						p.y = mouseY + ( Math.cos( temp ) * p.distToOrigin );
					}
					else
					{
						p.x = -1;
						p.y = -1;
					}	
					
					p.restLife = p.lifespan;
				}
				else
				{
					if( p.x != -1 && p.y != -1 )
					{
						p.y += p.restLife * p.viscosity;
						
						p.alpha = 0xFF * ( 1 - ( p.distToOrigin / _radius ) );
					
						p.paint( _canvas );
					}
				}
				
				p = p.next;
				
			} 
			while( p );
		}
	}
}
